#!/usr/bin/perl

#=== File Prolog =============================================================
#    Distributed.net Personal Proxy Statistics Database Interface
#    Copyright (C) 1998-2000 GNU General Public License
#    http://ppstats.sourceforge.net/
#    $Id: ppdb.pl,v 1.7 2003/02/26 12:52:09 work Exp $
#
#--- Contents ----------------------------------------------------------------
#    Perl interface routines for creating and maintaining the PostgreSQL
#    database used to store Distributed.Net personal proxy log files.
#
#--- Description -------------------------------------------------------------
#    A complete description of Personal Proxy Statistics can be found at
#    http://ppstats.sourceforge.net/
#
#    This script is used by the Personal Proxy Statistics administrator to
#    create and maintain the PostgreSQL database used by ppstats to store
#    the Distributed.Net personal proxy log files. The operations available
#    in ppdb are all available through the command line by passing arguments
#    to the script. See ppdb.pl -help for more information.
#
#    The database dictionary created by running ppdb.pl is located at
#    http://ppstats.sourceforge.net/doc/ddd.php3
#
#--- Notes -------------------------------------------------------------------
#    Before running this script, make sure you have met the system
#    requirements. A step by step guide to installing and initializing the
#    PostgreSQL database on Red Hat Linux is available at 
#    http://ppstats.sourceforge.net/docs/postgresql.php3
#
#    The template1 database that is created by PostgreSQL during setup is
#    required by this script and must be present.
#
#--- Warning -----------------------------------------------------------------
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of the GNU General Public License
#    as published by the Free Software Foundation; either version 2
#    of the License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#=== End File Prolog =========================================================


#--- Use Declarations --------------------------------------------------------
use strict;        #--Declare all variables.
use Time::Local;   #--Time is local.
use DBI;           #--Standard Perl database interface.
#use DBD::Pg;       #--Database driver for PostgreSQL.
use DBD::mysql;

#--- Global Variables --------------------------------------------------------
my %SystemPrefs;         #--Internal system variables.
my %opt;                 #--Command line switches.
my @optArgs;             #--Command line switch arguments.

#--Complete list of tables used by ppstats.
my %Tables = 
    ( "TBL_WU_RC564" => "TBL_WU_RC564.SQL",
      "TBL_WU_RC572" => "TBL_WU_RC572.SQL",
      "TBL_WU_DES" => "TBL_WU_DES.SQL",
      "TBL_WU_CSC" => "TBL_WU_CSC.SQL",
      "TBL_WU_OGR" => "TBL_WU_OGR.SQL",
      "TBL_OS" => "TBL_OS.SQL",
      "TBL_CPU" => "TBL_CPU.SQL",
      "TBL_HOST" => "TBL_HOST.SQL",
      "TBL_PARTICIPANT" => "TBL_PARTICIPANT.SQL",
      "TBL_COUNTRY" => "TBL_COUNTRY.SQL"
    );


#--- Method Prolog -----------------------------------------------------------
#  Name: Main Procedure
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Main procedure starts here. All work is done by the passing parameters
#    to this script.
#
#--- End Method Prolog -------------------------------------------------------
umask("002");
&SystemInit();
&ProcessCommandLine();
exit;


#--- Method Prolog -----------------------------------------------------------
#  Name: SystemInit
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Initialize internal script variables. Do not edit these settings unless
#    you really know what you are doing.
#
#--- End Method Prolog -------------------------------------------------------
sub SystemInit
{
    #--Set name, path, version, and debug variables.
    $SystemPrefs{"appname"} = "Personal Proxy Statistics Database Interface";
    $SystemPrefs{"appver"} = "8.1.1";
    ($SystemPrefs{"appdir"}, $SystemPrefs{"app"}) = split(m|([^/]+)$|, $0, 2);
    $SystemPrefs{"debugenable"} = 0;          #--Same as using the -debug switch.

    #--Database related variables.
    $SystemPrefs{"dbname"} = "ppstats";       #--The name of our ppstats database.
    $SystemPrefs{"template1"} = "template1";  #--The built in template database.
    $SystemPrefs{"dbh"} = "";                 #--This will be created in ConnectDB.

    #--Application variables.
    $SystemPrefs{"exportdir"} = "export/";    #--The default export directory.
    $SystemPrefs{"importdir"} = "import/";    #--The default import directory.
}  #--SystemInit


#--- Method Prolog -----------------------------------------------------------
#  Name: ProcessCommandLine
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Process command line arguements supplied by the user. Some parameters,
#    such as -import, -export, and -blacklist take an additional arguement.
#
#--- End Method Prolog -------------------------------------------------------
sub ProcessCommandLine
{
    #--Parse the command line variables.
    &ParseCommandLine(
	"-debug", 0,         #--Enable debug mode.
	"-version", 0,       #--Display version information.
	"-help", 0,          #--Display useage information.
        "-import", 1,        #--Import the given table from the import dir.
        "-importall", 0,     #--Import all tables from the import dir.
	"-export", 1,        #--Export the given table into the export dir.
	"-exportall", 0,     #--Export all tables into export dir.
        "-createdb", 0,      #--Create database
        "-createtables", 0,  #--Create database tables.
	"-createindexes", 0, #--Create database indexes.
        "-dropindexes", 0,   #--Drop database indexes.
	"-purgedb", 0,       #--Purge all data from database.
        "-updateuser", 1,    #--Update the given participant information.
        "-retire", 1,        #--Retire the given participant.
        "-createnetwork", 1, #--Create network for dhcp-ranges (one network instead of x-hosts)
        "-blacklist", 1      #--Blacklist this email address.
    );

    &EnableDebugMessages() if ($opt{"-debug"});
    &DisplayVersion() if ($opt{"-version"});
    &DisplayUsage() if ($opt{"-help"}); 

    &Import($opt{"-import"}) if ($opt{"-import"});         
    &ImportAll($opt{"-importall"}) if ($opt{"-importall"});

    &Export($opt{"-export"}) if ($opt{"-export"});
    &ExportAll() if ($opt{"-exportall"});

    &CreateDB() if ($opt{"-createdb"});
    &CreateTables() if ($opt{"-createtables"});
    &CreateIndexes() if ($opt{"-createindexes"});
    &DropIndexes() if ($opt{"-dropindexes"});
    &PurgeDB() if ($opt{"-purgedb"});

    &UpdateUser($opt{"-updateuser"}) if ($opt{"-updateuser"});
    &RetireEmail($opt{"-retire"}) if ($opt{"-retire"});
    &CreateNetwork($opt{"-createnetwork"}) if ($opt{"-createnetwork"});
    &BlacklistEmail($opt{"-blacklist"}) if ($opt{"-blacklist"});

    if (%opt == 0) {
	&DisplayUsage();
    }
}  #--ProcessCommandLine


#--- Method Prolog -----------------------------------------------------------
#  Name: SelectSQL
#
#  Parameters:
#    sql - The preformatted aggregate SQL statement to run.
#
#  Return Value:
#    sth - The resulting statement handle. Access the data in the recordset
#          with a command like: while (@row = $sth->fetchrow_array()) { ... }
#
#  Description:
#    Executes the specified SQL statement and return the results. This
#    function is for SELECT statements only. All other commands (such as
#    INSERT, DELETE, UPDATE)  should use RunSQL instead.
#
#--- End Method Prolog -------------------------------------------------------
sub SelectSQL
{
    my $sql = shift @_;

    my $sth = $SystemPrefs{"dbh"}->prepare($sql);
    $sth->execute() or die $DBI::errstr;

    return $sth;
}  #--SelectSQL


#--- Method Prolog -----------------------------------------------------------
#  Name: DLookup
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    This domain lookup provides a quick way to run a simple SQL statement 
#    that returns a single row of data.
#
#--- End Method Prolog -------------------------------------------------------
sub DLookup
{
    my $sql = shift @_;

    my $sth = &SelectSQL($sql);
    my @row = $sth->fetchrow_array();
    $sth->finish();
    return @row;
}  #--DLookup


#--- Method Prolog -----------------------------------------------------------
#  Name: RunSQL
#
#  Parameters:
#    sql - The preformatted aggregate SQL statement to run.
#
#  Return Value: None.
#
#  Description:
#    Executes the specified SQL data manipulation statement. The command
#    is executed and there should be no return value. SQL statements such as
#    INSERT, DELETE, and UPDATE should use this function, while SELECT
#    statements should call SelectSQL instead.
#
#--- End Method Prolog -------------------------------------------------------
sub RunSQL
{
    my $sql = shift @_;
    my $sth;

    $sth = $SystemPrefs{"dbh"}->prepare($sql);
    $sth->execute() or die $DBI::errstr;
    $sth->finish();   
}  #--RunSQL


#--- Method Prolog -----------------------------------------------------------
#  Name: ConnectDB
#
#  Parameters:
#    db - The name of the database to establish a connection to.
#
#  Return Value: None.
#
#  Description:
#    Connect to the database and create the database handle. Die otherwise.
#
#--- End Method Prolog -------------------------------------------------------
sub ConnectDB
{
    my $db = shift @_;

    &Debugln("Connecting to the database '$db' ...");
    
    $SystemPrefs{"dbh"} = DBI->connect(
        "dbi:mysql:dbname=" . $db, 
        "ppstats",
        "12345",
	{ AutoCommit => 1, RaiseError => 0 }
    );
    die $DBI::errstr unless defined($SystemPrefs{"dbh"});

    #--Set the default time zone for all datetime data types to GMT.
#    &RunSQL("SET TIMEZONE TO 'GMT'");
#    &RunSQL("SET DATESTYLE TO 'ISO'");

    &Debugln("Connected to database.");       	
}  #--ConnectDB


#--- Method Prolog -----------------------------------------------------------
#  Name: DisconnectDB
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Disconnect from the database.
#
#--- End Method Prolog -------------------------------------------------------
sub DisconnectDB
{
    $SystemPrefs{"dbh"}->disconnect();
    &Debugln("Disconnected from the database.");       	
}  #--DisconnectDB


#--- Method Prolog -----------------------------------------------------------
#  Name: CreateDB
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Create the Personal Proxy Statistics database.
#
#--- End Method Prolog -------------------------------------------------------
sub CreateDB
{
    my $sql;

    #--Connect to the default template database to establish a connection.
    &ConnectDB($SystemPrefs{"template1"});

    #--Create the ppstats database.
    &Debugln("Creating '" . $SystemPrefs{"dbname"} . "' database ...");
    $sql = "CREATE DATABASE " . $SystemPrefs{"dbname"};
    &RunSQL($sql);
    &Debugln("Database '" . $SystemPrefs{"dbname"} . "' created.");
    &DisconnectDB();

    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n",
          "Database created successfully.\n\n",
          "If you are running Personal Proxy Statistics for the first time,\n",
          "execute the following command to create the required tables:\n",
          "  ppdb.pl -createtables\n\n";

    exit;
}  #--CreateDB


#--- Method Prolog -----------------------------------------------------------
#  Name: CreateTables
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Create all the necessary database tables. Indexes are created only for
#    the static tables. CreateIndexes will create indexes on the much larger
#    Work Unit tables. Similarly, DropIndexes will drop indexes for them.
#
#    See http://ppstats.sourceforge.net/docs/ddd.php3 for data dictionary.
#
#--- End Method Prolog -------------------------------------------------------
sub CreateTables
{
    my $sql;

    #--Connect to the database and create database objects.
    &ConnectDB($SystemPrefs{"dbname"});
    &Debugln("Creating database tables ...");


    #--Create the table structure for TBL_WU_RC564.
    $sql = "CREATE TABLE TBL_WU_RC564 (\n" .
           " SUBMITTED_DATE DATE NOT NULL,\n" .
           " SUBMITTED_TIME TIME NOT NULL,\n" .
           " HOST_ID        INT2 NOT NULL,\n" .
           " PARTICIPANT_ID INT2 NOT NULL,\n" .
           " WU_SIZE        INT4 NOT NULL,\n" .
           " OS_ID          INT2 NOT NULL,\n" .
           " CPU_ID         INT2 NOT NULL,\n" .
           " VERSION        VARCHAR(4) NOT NULL\n" .
           ")";                                               
    &RunSQL($sql);


    #--Create the table structure for TBL_WU_RC572.
    $sql = "CREATE TABLE TBL_WU_RC572 (\n" .
           " SUBMITTED_DATE DATE NOT NULL,\n" .
           " SUBMITTED_TIME TIME NOT NULL,\n" .
           " HOST_ID        INT2 NOT NULL,\n" .
           " PARTICIPANT_ID INT2 NOT NULL,\n" .
           " WU_SIZE        INT4 NOT NULL,\n" .
           " OS_ID          INT2 NOT NULL,\n" .
           " CPU_ID         INT2 NOT NULL,\n" .
           " VERSION        VARCHAR(4) NOT NULL\n" .
           ")";                                               
    &RunSQL($sql);


    #--Create the table structure for TBL_WU_DES.
    $sql = "CREATE TABLE TBL_WU_DES (\n" .
           " SUBMITTED_DATE DATE NOT NULL,\n" .
           " SUBMITTED_TIME TIME NOT NULL,\n" .
           " HOST_ID        INT2 NOT NULL,\n" .
           " PARTICIPANT_ID INT2 NOT NULL,\n" .
           " WU_SIZE        INT4 NOT NULL,\n" .
           " OS_ID          INT2 NOT NULL,\n" .
           " CPU_ID         INT2 NOT NULL,\n" .
           " VERSION        VARCHAR(4) NOT NULL\n" .
           ")";                                               
    &RunSQL($sql);


    #--Create the table structure for TBL_WU_CSC.
    $sql = "CREATE TABLE TBL_WU_CSC (\n" .
           " SUBMITTED_DATE DATE NOT NULL,\n" .
           " SUBMITTED_TIME TIME NOT NULL,\n" .
           " HOST_ID        INT2 NOT NULL,\n" .
           " PARTICIPANT_ID INT2 NOT NULL,\n" .
           " WU_SIZE        INT4 NOT NULL,\n" .
           " OS_ID          INT2 NOT NULL,\n" .
           " CPU_ID         INT2 NOT NULL,\n" .
           " VERSION        VARCHAR(4) NOT NULL\n" .
           ")";                                               
    &RunSQL($sql);


    #--Create the table structure for TBL_WU_OGR.
    $sql = "CREATE TABLE TBL_WU_OGR (\n" .
           " SUBMITTED_DATE DATE NOT NULL,\n" .
           " SUBMITTED_TIME TIME NOT NULL,\n" .
           " HOST_ID        INT2 NOT NULL,\n" .
           " PARTICIPANT_ID INT2 NOT NULL,\n" .
           " WU_SIZE        INT8 NOT NULL,\n" .
           " OS_ID          INT2 NOT NULL,\n" .
           " CPU_ID         INT2 NOT NULL,\n" .
           " VERSION        VARCHAR(4) NOT NULL\n" .
           ")";                                               
    &RunSQL($sql);


    #--Create the table structure for TBL_OS.
    $sql = "CREATE TABLE TBL_OS (\n" .
           " OS_ID   INT2 PRIMARY KEY,\n" .
           " OS      VARCHAR(30) NOT NULL,\n" .
           " OS_ICON VARCHAR(30) NOT NULL\n" .   
           ")";                                               
    &RunSQL($sql);


    #--Create the table structure for TBL_CPU.
    $sql = "CREATE TABLE TBL_CPU (\n" .
           " CPU_ID   INT2 PRIMARY KEY,\n" .
           " CPU      VARCHAR(30) NOT NULL,\n" .
           " CPU_ICON VARCHAR(30) NOT NULL\n" .   
           ")";                                               
    &RunSQL($sql);


    #--Create the table structure for TBL_HOST.
    $sql = "CREATE TABLE TBL_HOST (\n" .
           " HOST_ID    INT2 PRIMARY KEY,\n" .
           " IP         VARCHAR(16) NOT NULL,\n" .
           " NETMASK    VARCHAR(16) NOT NULL default '255.255.255.255',\n" .
           " HOSTNAME   VARCHAR(50),\n" .   
           " COUNTRY_ID VARCHAR(5),\n" .   
           " UPDATED    DATETIME\n" .    
           ")";                                               
    &RunSQL($sql);

    $sql = "CREATE UNIQUE INDEX TBL_HOST_IP_IDX ON TBL_HOST (IP)";
    &RunSQL($sql);


    #--Create the table structure for TBL_PARTICIPANT.
    $sql = "CREATE TABLE TBL_PARTICIPANT (\n" .
           " PARTICIPANT_ID INT2 PRIMARY KEY,\n" .
           " EMAIL          VARCHAR(50) NOT NULL,\n" .
           " DNET_ID        VARCHAR(20),\n" .   
           " FNAME          VARCHAR(30),\n" .    
           " LNAME          VARCHAR(30),\n" .
           " NICKNAME       VARCHAR(30),\n" .
           " DOB            DATE,\n" .
           " HOMEPAGE       VARCHAR(250),\n" .        
           " COUNTRY_ID     VARCHAR(5) NOT NULL,\n" .        
           " MEMBER         TINYINT NOT NULL,\n" . 
           " RETIRE_TO_ID   INT2\n" .       
           ")";                                               
    &RunSQL($sql);

    $sql = "CREATE UNIQUE INDEX TBL_PARTICIPANT_EMAIL_IDX ON TBL_PARTICIPANT (EMAIL)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_PARTICIPANT_COUNTRY_ID_IDX ON TBL_PARTICIPANT (COUNTRY_ID)";
    &RunSQL($sql);


    #--Create the table structure for TBL_COUNTRY.
    $sql = "CREATE TABLE TBL_COUNTRY (\n" .
           " COUNTRY_ID   VARCHAR(5) PRIMARY KEY,\n" .
           " COUNTRY      VARCHAR(50) NOT NULL,\n" .
           " COUNTRY_ICON VARCHAR(30) NOT NULL\n" .   
           ")";                                               
    &RunSQL($sql);


    &Debugln("Tables created.");
    &DisconnectDB();

    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n",
          "Database tables created successfully.\n\n",
          "If you are running Personal Proxy Statistics for the first time, or if\n",
          "you are rebuilding from an existing database, please copy any backup\n",
          "SQL data files into the import directory, and then execute the command:\n",
          "  ppdb.pl -importall\n\n";

    exit;
}  #--CreateTables


#--- Method Prolog -----------------------------------------------------------
#  Name: CreateIndexes
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Creates indexes for the larger Work Unit tables. Adding indexes only
#    after the first import has been completed greatly increases the speed of
#    the initial log import.
#
#    See http://ppstats.sourceforge.net/docs/ddd.php3 for data dictionary.
#
#--- End Method Prolog -------------------------------------------------------
sub CreateIndexes
{
    my $sql;

    #--Connect to the database and create database objects.
    &ConnectDB($SystemPrefs{"dbname"});
    &Debugln("Creating database indexes ...");


    #--Create the indexes for TBL_WU_RC564.
    $sql = "CREATE INDEX TBL_WU_RC564_SUBMITTED_DATE_IDX ON TBL_WU_RC564 (SUBMITTED_DATE)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_RC564_SUBMITTED_TIME_IDX ON TBL_WU_RC564 (SUBMITTED_TIME)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_RC564_HOST_ID_IDX ON TBL_WU_RC564 (HOST_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_RC564_PARTICIPANT_ID_IDX ON TBL_WU_RC564 (PARTICIPANT_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_RC564_OS_ID_IDX ON TBL_WU_RC564 (OS_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_RC564_CPU_ID_IDX ON TBL_WU_RC564 (CPU_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_RC564_VERSION_IDX ON TBL_WU_RC564 (VERSION)";
    &RunSQL($sql);


    #--Create the indexes for TBL_WU_RC572.
    $sql = "CREATE INDEX TBL_WU_RC572_SUBMITTED_DATE_IDX ON TBL_WU_RC572 (SUBMITTED_DATE)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_RC572_SUBMITTED_TIME_IDX ON TBL_WU_RC572 (SUBMITTED_TIME)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_RC572_HOST_ID_IDX ON TBL_WU_RC572 (HOST_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_RC572_PARTICIPANT_ID_IDX ON TBL_WU_RC572 (PARTICIPANT_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_RC572_OS_ID_IDX ON TBL_WU_RC572 (OS_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_RC572_CPU_ID_IDX ON TBL_WU_RC572 (CPU_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_RC572_VERSION_IDX ON TBL_WU_RC572 (VERSION)";
    &RunSQL($sql);


    #--Create the indexes for TBL_WU_DES.
    $sql = "CREATE INDEX TBL_WU_DES_SUBMITTED_DATE_IDX ON TBL_WU_DES (SUBMITTED_DATE)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_DES_SUBMITTED_TIME_IDX ON TBL_WU_DES (SUBMITTED_DATE)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_DES_HOST_ID_IDX ON TBL_WU_DES (HOST_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_DES_PARTICIPANT_ID_IDX ON TBL_WU_DES (PARTICIPANT_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_DES_OS_ID_IDX ON TBL_WU_DES (OS_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_DES_CPU_ID_IDX ON TBL_WU_DES (CPU_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_DES_VERSION_IDX ON TBL_WU_DES (VERSION)";
    &RunSQL($sql);


    #--Create the indexes for TBL_WU_CSC.
    $sql = "CREATE INDEX TBL_WU_CSC_SUBMITTED_DATE_IDX ON TBL_WU_CSC (SUBMITTED_DATE)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_CSC_SUBMITTED_TIME_IDX ON TBL_WU_CSC (SUBMITTED_TIME)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_CSC_HOST_ID_IDX ON TBL_WU_CSC (HOST_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_CSC_PARTICIPANT_ID_IDX ON TBL_WU_CSC (PARTICIPANT_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_CSC_OS_ID_IDX ON TBL_WU_CSC (OS_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_CSC_CPU_ID_IDX ON TBL_WU_CSC (CPU_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_CSC_VERSION_IDX ON TBL_WU_CSC (VERSION)";
    &RunSQL($sql);


    #--Create the indexes for TBL_WU_OGR.
    $sql = "CREATE INDEX TBL_WU_OGR_SUBMITTED_DATE_IDX ON TBL_WU_OGR (SUBMITTED_DATE)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_OGR_SUBMITTED_TIME_IDX ON TBL_WU_OGR (SUBMITTED_TIME)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_OGR_HOST_ID_IDX ON TBL_WU_OGR (HOST_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_OGR_PARTICIPANT_ID_IDX ON TBL_WU_OGR (PARTICIPANT_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_OGR_OS_ID_IDX ON TBL_WU_OGR (OS_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_OGR_CPU_ID_IDX ON TBL_WU_OGR (CPU_ID)";
    &RunSQL($sql);

    $sql = "CREATE INDEX TBL_WU_OGR_VERSION_IDX ON TBL_WU_OGR (VERSION)";
    &RunSQL($sql);


    &Debugln("Indexes created.");

    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n",
          "Database indexes created successfully.\n\n";

    exit;
}  #--CreateIndexes


#--- Method Prolog -----------------------------------------------------------
#  Name: DropIndexes
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Drops indexes for the larger Work Unit tables. 
#
#    See http://ppstats.sourceforge.net/docs/ddd.php3 for data dictionary.
#
#--- End Method Prolog -------------------------------------------------------
sub DropIndexes
{
    my $sql;

    #--Connect to the database and create database objects.
    &ConnectDB($SystemPrefs{"dbname"});
    &Debugln("Dropping database indexes ...");


    #--Drop the indexes for TBL_WU_RC564.
    $sql = "DROP INDEX TBL_WU_RC564_SUBMITTED_DATE_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_RC564_SUBMITTED_TIME_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_RC564_HOST_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_RC564_PARTICIPANT_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_RC564_OS_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_RC564_CPU_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_RC564_VERSION_IDX";
    &RunSQL($sql);


    #--Drop the indexes for TBL_WU_DES.
    $sql = "DROP INDEX TBL_WU_DES_SUBMITTED_DATE_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_DES_SUBMITTED_TIME_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_DES_HOST_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_DES_PARTICIPANT_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_DES_OS_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_DES_CPU_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_DES_VERSION_IDX";
    &RunSQL($sql);


    #--Drop the indexes for TBL_WU_CSC.
    $sql = "DROP INDEX TBL_WU_CSC_SUBMITTED_DATE_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_CSC_SUBMITTED_TIME_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_CSC_HOST_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_CSC_PARTICIPANT_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_CSC_OS_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_CSC_CPU_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_CSC_VERSION_IDX";
    &RunSQL($sql);


    #--Drop the indexes for TBL_WU_OGR.
    $sql = "DROP INDEX TBL_WU_OGR_SUBMITTED_DATE_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_OGR_SUBMITTED_TIME_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_OGR_HOST_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_OGR_PARTICIPANT_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_OGR_OS_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_OGR_CPU_ID_IDX";
    &RunSQL($sql);

    $sql = "DROP INDEX TBL_WU_OGR_VERSION_IDX";
    &RunSQL($sql);


    &Debugln("Indexes dropped.");
    &DisconnectDB();

    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n",
          "Database indexes dropped successfully.\n\n";

    exit;
}  #--DropIndexes


#--- Method Prolog -----------------------------------------------------------
#  Name: PurgeDB
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Purge all the database objects. Dropping a table drops its indexes as
#    well. CreateDB must be called before another data import occurs.
#
#    WARNING: There is no way to undo a purged database, so be sure to have
#    backups of your data using the -exportall arguement first.
#
#--- End Method Prolog -------------------------------------------------------
sub PurgeDB
{
    my $sql;
    my $result;

    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n\n",
          "WARNING: There is no way to undo a purged database, so be sure to have\n",
          "backups of your data using the -exportall argument first.\n\n";

    print "Are you sure you want to purge the '" . $SystemPrefs{"dbname"} . "' database? [N/y]: ";
    $result = <STDIN>;
    chomp($result);
    $result =~ tr/A-Z/a-z/;
    print "\n";

    if ($result eq "y") {
	#--Connect to the default template database to establish a connection.
	&ConnectDB($SystemPrefs{"template1"});

	#--Drop the database first, and don't complain if its not there.
	&Debugln("Dropping '" . $SystemPrefs{"dbname"} . "' database if present ... ");
	$sql = "DROP DATABASE " . $SystemPrefs{"dbname"};
	&RunSQL($sql);
	&Debugln("Database '" . $SystemPrefs{"dbname"} . "' dropped.");
	&DisconnectDB();
    
	print "Database purged successfully. You can recreate the database by\n",
	      "executing the commands:\n",
              "  ppdb.pl -createdb\n",
              "  ppdb.pl -createtables\n",
              "  ppdb.pl -importall\n",
	      "  ppstats.pl -importonly -debug\n",
              "  ppdb.pl -createindexes\n\n";
    } else {
	print "Database purge aborted.\n\n";
    }

    exit;
}  #--PurgeDB


#--- Method Prolog -----------------------------------------------------------
#  Name: UpdateUser
#
#  Parameters:
#    DataFile - The file which contains the participant information.
#
#  Return Value: None.
#
#  Description:
#    This function is called by the script administrator who supplies the
#    path to an ini-file. The file contains a list of values that users
#    will email such as their first and last names, homepage url, etc.
#
#    Note that the ini file will only accept values for the following
#    fields (case sensitive data):
#      email (required), dnet_id, fname, lname, nickname, dob, homepage,
#      and country_id.
#
#--- End Method Prolog -------------------------------------------------------
sub UpdateUser
{
    my $DataFile = shift @_;
    my $entry;
    my ($var, $value);
    my $sql;
    my %DataFields;
    my $field;
    my $comma;
    my $participant_id;

    #--This is a list of fields that can be updated by this procedure.
    my @AvailableFields = ( 
        "email", "dnet_id", "fname", "lname", 
	"nickname", "dob", "homepage", "country_id" 
    );

    &Debugln("Updating participant table with data from '$DataFile' ...");

    open(DATAFILE, $DataFile) or die $SystemPrefs{"appname"} . " : Cannot open " . $DataFile . " ($!)\n";
    while (defined($entry = <DATAFILE>)) {
        chomp($entry);               #--no newline
        $entry =~ s/#.*//;           #--no comments
        $entry =~ s/^\s+//;          #--no leading white
        $entry =~ s/\s+$//;          #--no trailing white
        next unless length($entry);  #--anything left?
        ($var, $value) = split(/\s*=\s*/, $entry, 2);
        $value =~ s/\"//g;           #--remove "'s
#        $var =~ tr/A-Z/a-z/;         #--convert to lowercase
        $DataFields{$var} = $value   #--set var to value         
    }
    close DATAFILE;

    &ConnectDB($SystemPrefs{"dbname"});

    #--The one required field is the email address.
    if (defined($DataFields{"email"})) {
	#--Get the old email address participant ID.
	$sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE EMAIL='" . $DataFields{"email"} . "'"; 
	($participant_id) = &DLookup($sql);
    }

    #--Make sure the email address existed in the database.
    if (defined($participant_id)) {
	$sql = "UPDATE TBL_PARTICIPANT SET ";
	$comma = "";
	foreach $field (@AvailableFields) {
	    if (defined($DataFields{$field})) {
		$sql .= $comma . "$field='" . $DataFields{$field} ."'";
		$comma = ","
	    }
	}
	$sql .= " WHERE PARTICIPANT_ID=" . $participant_id;

	&RunSQL($sql);
	print "Update user successful.\n";
    } else {
	print "Update user aborted. Make sure the 'email' field was supplied in the\n",
              "file '" . $DataFile . "' and is present in the database.\n\n";
    }

    &DisconnectDB();

    exit;
}  #--UpdateUser


#--- Method Prolog -----------------------------------------------------------
#  Name: RetireEmail
#
#  Parameters:
#    DataFile - The file which contains the participant information.
#
#  Return Value: None.
#
#  Description:
#    This function is called by the script administrator who supplies the
#    path to an ini-file. The file contains the participants old email
#    address, and the new email address. All work units credited to the old
#    email address will be updated to the new address. All future work
#    done will be credited towards the new email address as well.
#
#--- End Method Prolog -------------------------------------------------------
sub RetireEmail
{
    my $DataFile = shift @_;
    my $entry;
    my ($var, $value);
    my $sql;
    my %DataFields;
    my $oldemail_id;
    my $newemail_id;
    my $result;
    my @row;

    open(DATAFILE, $DataFile) or die $SystemPrefs{"appname"} . " : Cannot open " . $DataFile . " ($!)\n";
    while (defined($entry = <DATAFILE>)) {
        chomp($entry);               #--no newline
        $entry =~ s/#.*//;           #--no comments
        $entry =~ s/^\s+//;          #--no leading white
        $entry =~ s/\s+$//;          #--no trailing white
        next unless length($entry);  #--anything left?
        ($var, $value) = split(/\s*=\s*/, $entry, 2);
        $value =~ s/\"//g;           #--remove "'s
        $var =~ tr/A-Z/a-z/;         #--convert to lowercase
        $DataFields{$var} = $value   #--set var to value         
    }
    close DATAFILE;

    &ConnectDB($SystemPrefs{"dbname"});

    #--Verify that both an old and new address were supplied.
    if (defined($DataFields{"oldemail"}) && defined($DataFields{"newemail"})) {
	#--Get the old email address participant ID.
	$sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE EMAIL='" . $DataFields{"oldemail"} . "'"; 
	($oldemail_id) = &DLookup($sql);

	#--Get the new email address participant ID.
	$sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE EMAIL='" . $DataFields{"newemail"} . "'"; 
	($newemail_id) = &DLookup($sql);
    }

    #--If we have both the old and new ID's, proceed with the update.
    if (defined($oldemail_id) && defined($newemail_id)) {
	print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n\n",
              "WARNING: Only a complete purge and rebuild of the database from the\n",
              "original log files can fully un-retire an email address.\n\n";

	print "Are you sure you want to retire the old email address\n" . 
              "of '" . $DataFields{"oldemail"} . "' to '" . $DataFields{"newemail"} . "'? [N/y]: ";
	$result = <STDIN>;
	chomp($result);
	$result =~ tr/A-Z/a-z/;
	print "\n";

	if ($result eq "y") {
	    #--Update the RETIRE_TO_ID field.
	    $sql = "UPDATE TBL_PARTICIPANT SET RETIRE_TO_ID=$newemail_id WHERE PARTICIPANT_ID=$oldemail_id";
	    &RunSQL($sql);

	    #--Update the Work Unit tables.
	    $sql = "UPDATE TBL_WU_RC564 SET PARTICIPANT_ID=$newemail_id WHERE PARTICIPANT_ID=$oldemail_id";
	    &RunSQL($sql);

	    $sql = "UPDATE TBL_WU_DES SET PARTICIPANT_ID=$newemail_id WHERE PARTICIPANT_ID=$oldemail_id";
	    &RunSQL($sql);

	    $sql = "UPDATE TBL_WU_CSC SET PARTICIPANT_ID=$newemail_id WHERE PARTICIPANT_ID=$oldemail_id";
	    &RunSQL($sql);

	    $sql = "UPDATE TBL_WU_OGR SET PARTICIPANT_ID=$newemail_id WHERE PARTICIPANT_ID=$oldemail_id";
	    &RunSQL($sql);

	    print "Email retirement successful.\n\n";
	}	
    } else {
	print "Email retirement aborted. Make sure both OldEmail and NewEmail addresses are\n" .
              "specified in '" . $DataFile . "' and are present in the database.\n\n";
    }

    &DisconnectDB();

    exit;
}  #--RetireEmail


#--- Method Prolog -----------------------------------------------------------
#  Name: CreateNetwork
#
#  Parameters:
#    DataFile - The file which contains the participant information.
#
#  Return Value: None.
#
#  Description:
#    This function is called by the script administrator who supplies the
#    path to an ini-file. The file contains the network and subnetmask,
#    and the name for this network. All work units submitted by one ip 
#    within this range will be counted for the network (as if the "network"
#    is one single "host")
#
#--- End Method Prolog -------------------------------------------------------
sub CreateNetwork
{
    my $DataFile = shift @_;
    my $entry;
    my ($var, $value);
    my $sql;
    my %DataFields;
    my $network_ip;
    my $network_mask;
    my $network_name;
    my $network_country;
    my $result;
    my @row;
    my $host_id;

    open(DATAFILE, $DataFile) or die $SystemPrefs{"appname"} . " : Cannot open " . $DataFile . " ($!)\n";
    while (defined($entry = <DATAFILE>)) {
        chomp($entry);               #--no newline
        $entry =~ s/#.*//;           #--no comments
        $entry =~ s/^\s+//;          #--no leading white
        $entry =~ s/\s+$//;          #--no trailing white
        next unless length($entry);  #--anything left?
        ($var, $value) = split(/\s*=\s*/, $entry, 2);
        $value =~ s/\"//g;           #--remove "'s
#        $var =~ tr/A-Z/a-z/;         #--convert to lowercase
        $DataFields{$var} = $value   #--set var to value         
    }
    close DATAFILE;

    &ConnectDB($SystemPrefs{"dbname"});

    #--Verify that both an ip and netmask were supplied.
    if (defined($DataFields{"ip"}) && defined($DataFields{"mask"})) 
    {
	$network_ip = $DataFields{"ip"};
	$network_mask = $DataFields{"mask"};
    }
    
    if (defined($DataFields{"name"}))
    {
	$network_name = $DataFields{"name"};
    }
    else
    {
	$network_name = $network_ip . "/" . $network_mask;
    }

    if (defined($DataFields{"country"}))
    {
	$network_country = $DataFields{"country"};
    }
    else
    {
	$network_country = "local";
    }
    
    #--If we have both the old and new ID's, proceed with the update.
    if (defined($network_ip) && defined($network_mask)) 
    {
	print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n\n",
              "WARNING: Only a complete purge and rebuild of the database from the\n",
              "original log files can fully remove this network entry\n\n";

	print "Are you sure you want to add the network " . $DataFields{"name"} . "\n" . 
              "'" . $DataFields{"ip"} . "/" . $DataFields{"mask"} . "' with the country '" . $DataFields{"country"} . "'? [N/y]: ";
	$result = <STDIN>;
	chomp($result);
	$result =~ tr/A-Z/a-z/;
	print "\n";

	if ($result eq "y") 
	{
	    #--Get currently highest host_id
	    $sql = "SELECT MAX(HOST_ID) AS MAX_HOST_ID FROM TBL_HOST";
	    ($host_id) = &DLookup($sql);
	    if (defined($host_id)) 
	    {
		$host_id++;
	    } 
	    else 
	    {
		$host_id = 1;
	    }

	    #--Add network to TBL_HOST 
	    $sql = "INSERT INTO TBL_HOST (HOST_ID, IP, NETMASK, HOSTNAME, COUNTRY_ID, UPDATED) VALUES ('$host_id', '$network_ip', '$network_mask', '$network_name', '$network_country', '0000-00-00 00:00:00')";
	    &RunSQL($sql);

#	    #--Update the Work Unit tables.
#	    $sql = "UPDATE TBL_WU_RC564 SET PARTICIPANT_ID=$newemail_id WHERE PARTICIPANT_ID=$oldemail_id";
#	    &RunSQL($sql);
#
#	    $sql = "UPDATE TBL_WU_DES SET PARTICIPANT_ID=$newemail_id WHERE PARTICIPANT_ID=$oldemail_id";
#	    &RunSQL($sql);
#
#	    $sql = "UPDATE TBL_WU_CSC SET PARTICIPANT_ID=$newemail_id WHERE PARTICIPANT_ID=$oldemail_id";
#	    &RunSQL($sql);
#
#	    $sql = "UPDATE TBL_WU_OGR SET PARTICIPANT_ID=$newemail_id WHERE PARTICIPANT_ID=$oldemail_id";
#	    &RunSQL($sql);

	    print "Creating of network entry successful.\n\n";
	}	
    } else {
	print "Creating of network entry aborted. Make sure no host in the network\n" .
              "specified in '" . $DataFile . "' and are present in the database.\n\n";
    }

    &DisconnectDB();

    exit;
}  #--CreateNetwork


#--- Method Prolog -----------------------------------------------------------
#  Name: BlacklistEmail
#
#  Parameters:
#    email - The email address to be banned from the database.
#
#  Return Value: None.
#
#  Description:
#    This function allows ppstats administrators to ban or "blacklist" a
#    specified email address from being counted in stats runs. The given
#    email address is marked in the participants table as being banned, and
#    all work units credited to that email address are perminantly removed.
#
#    WARNING: Only a complete purge and rebuild of the database from the
#    original log files can un-blacklist an email address.
#
#--- End Method Prolog -------------------------------------------------------
sub BlacklistEmail
{
    my $email = shift @_;
    my $result;
    my $sql;
    my $rst;
    my @row;
    my $participant_id;

    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n\n",
          "WARNING: Only a complete purge and rebuild of the database from the\n",
          "original log files can un-blacklist an email address.\n\n";

    print "Are you sure you want to blacklist '$email'? [N/y]: ";
    $result = <STDIN>;
    chomp($result);
    $result =~ tr/A-Z/a-z/;
    print "\n";
    
    #--Need a working connection for this.
    &ConnectDB($SystemPrefs{"dbname"});  

    if ($result eq "y") {
	&Debugln("Blacklisting email address '" . $email . "'.");

	$sql = "SELECT PARTICIPANT_ID FROM TBL_PARTICIPANT WHERE EMAIL='" . $email . "'"; 
	($participant_id) = &DLookup($sql);

	if (defined($participant_id)) {
	    &Debugln("Purging email '" . $email . "' (" . $participant_id . ") from database.");

	    #--Mark this email address as no longer a member.
	    $sql = "UPDATE TBL_PARTICIPANT SET MEMBER=FALSE WHERE PARTICIPANT_ID=" . $participant_id;
	    &RunSQL($sql);

	    #--Purge all work units submitted by this email address.
	    $sql = "DELETE FROM TBL_WU_RC564 WHERE PARTICIPANT_ID=" . $participant_id;
	    &RunSQL($sql);
	    $sql = "DELETE FROM TBL_WU_DES WHERE PARTICIPANT_ID=" . $participant_id;
	    &RunSQL($sql);
	    $sql = "DELETE FROM TBL_WU_CSC WHERE PARTICIPANT_ID=" . $participant_id;
	    &RunSQL($sql);
	    $sql = "DELETE FROM TBL_WU_OGR WHERE PARTICIPANT_ID=" . $participant_id;
	    &RunSQL($sql);
	}
	print "Blacklist email successful.\n\n";
    } else {
	print "Blacklist email aborted.\n\n";
    }

    &DisconnectDB();

    exit;
}  #--BlacklistEmail


#--- Method Prolog -----------------------------------------------------------
#  Name: Import
#
#  Parameters:
#    Table - The table name to import.
#
#  Return Value: None.
#
#  Description:
#    Given a table name, this function will read in the appropriate file from
#    the import directory and insert the data into the database table.
#
#--- End Method Prolog -------------------------------------------------------
sub Import
{
    my $Table = shift @_;
    my $ImportDir = $SystemPrefs{"appdir"} . $SystemPrefs{"importdir"};

    &ConnectDB($SystemPrefs{"dbname"});

    $Table =~ tr/a-z/A-Z/;         #--convert to uppercase
    if (defined($Tables{$Table})) {
	&ImportTable($Table, $ImportDir . $Tables{$Table});
    } else {
	die("The specified table '$Table' is unknown and can not be imported.\n");
    }

    &DisconnectDB();

    exit;
}  #--Import


#--- Method Prolog -----------------------------------------------------------
#  Name: ImportAll
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Import all known tables into the database from data files in the 
#    import directory.
#
#--- End Method Prolog -------------------------------------------------------
sub ImportAll
{
    my $ImportDir = $SystemPrefs{"appdir"} . $SystemPrefs{"importdir"};
    my $Table;

    &ConnectDB($SystemPrefs{"dbname"});

    #--Dump out each of the defined tables.
    foreach $Table (keys %Tables) {
	&ImportTable($Table, $ImportDir . $Tables{$Table});
    }

    &DisconnectDB();

    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n",
          "Table data imported successfully.\n\n",
          "Personal Proxy log file data can be imported by executing the\n",
          "Personal Proxy Statistics Log Analyzer script, ppstats.pl\n",
          "Please note that it is highly recommended that you use the -importonly switch\n",
          "the first time importing data with ppstats.pl The import procedure can\n",
          "take anywhere from a few minutes to several hours depending on the size\n",
          "and number of your log files. Subsequent imports will only scan unread\n",
          "log files and therefore execute much quicker. After the first data import is\n",
          "complete, you must create indexes on the database to speed subsequent runs.\n",
          "The commands to execute are as follows:\n",
          "  ppstats.pl -importonly -debug\n",
          "  ppdb.pl -createindexes\n",
          "  ppstats.pl -debug\n\n";

    exit;
}  #--ImportAll


#--- Method Prolog -----------------------------------------------------------
#  Name: ImportTable
#
#  Parameters:
#    Table - The table name to import.
#    InFile - The filename that contains the data to import.
#
#  Return Value: None.
#
#  Description:
#    This is a helper function that does the actual work of reading in data
#    from the given file into the given table name.
#
#--- End Method Prolog -------------------------------------------------------
sub ImportTable
{
    my ($Table, $InFile) = @_;
    my $sql;

    &Debugln("Importing '$Table' from file '$InFile' ...");

    open(INFILE, $InFile) or die $SystemPrefs{"appname"} . " : Cannot open " . $InFile . " ($!)\n";
    while (defined($sql = <INFILE>)) {
	chomp($sql);
	&RunSQL($sql);
    }
    close INFILE;
}  #--ImportTable


#--- Method Prolog -----------------------------------------------------------
#  Name: Export
#
#  Parameters:
#    Table - The table name to export.
#
#  Return Value: None.
#
#  Description:
#    Given a table name, this function will export the table data into the 
#    export directory.
#
#--- End Method Prolog -------------------------------------------------------
sub Export
{
    my $Table = shift @_;
    my $ExportDir = $SystemPrefs{"appdir"} . $SystemPrefs{"exportdir"};

    &ConnectDB($SystemPrefs{"dbname"});

    $Table =~ tr/a-z/A-Z/;         #--convert to uppercase
    if (defined($Tables{$Table})) {
	&ExportTable($Table, $ExportDir . $Tables{$Table});
    } else {
	die("The specified table '$Table' is unknown and can not be exported.\n");
    }

    &DisconnectDB();

    exit;
}  #--Export


#--- Method Prolog -----------------------------------------------------------
#  Name: ExportAll
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Export all known tables from the database into data files in the 
#    export directory.
#
#--- End Method Prolog -------------------------------------------------------
sub ExportAll
{
    my $ExportDir = $SystemPrefs{"appdir"} . $SystemPrefs{"exportdir"};
    my $Table;

    &ConnectDB($SystemPrefs{"dbname"});

    #--Dump out each of the defined tables.
    foreach $Table (keys %Tables) {
	&ExportTable($Table, $ExportDir . $Tables{$Table});
    }

    &DisconnectDB();

    exit;
}  #--ExportAll


#--- Method Prolog -----------------------------------------------------------
#  Name: ExportTable
#
#  Parameters:
#    Table - The table name to export.
#    OutFile - The filename to export data into.
#
#  Return Value: None.
#
#  Description:
#    This is a helper function that does the actual work of writing out data
#    to the given file from the given table name.
#
#--- End Method Prolog -------------------------------------------------------
sub ExportTable
{
    my ($Table, $OutFile) = @_;
    my $sql;
    my $k;
    my $rst;
    my @row;
    my $fields;
    my $line;
    my $comma;

    &Debugln("Exporting '$Table' to file '$OutFile' ...");

    open(OUTFILE, ">" . $OutFile) or die $SystemPrefs{"appname"} . " : Cannot open " . $OutFile . " ($!)\n";

    $sql = "SELECT * FROM $Table";
    $rst = &SelectSQL($sql);

    $comma = "";
    for ($k = 0; $k < $rst->nfields; $k++) {
	$fields .= $comma . $rst->fname($k);
	$comma = ",";
	&Debugln(sprintf("\tfield = %-12.12s type = >%s<", $rst->fname($k), $rst->ftype($k)));
    }

    while (@row = $rst->fetchrow) {
	$comma = "";
	$line = "INSERT INTO $Table ($fields) VALUES (";
	for ($k = 0; $k < $rst->nfields; $k++) {
	    $line .= $comma;
	    if (!defined($row[$k])) {
		$line .=  "NULL";
	    } elsif ($rst->ftype($k) == 16) {
		if ($row[$k] eq "f") {
		    $line .= "FALSE";
		} else {
		    $line .= "TRUE";		    
		}
	    } elsif ($rst->ftype($k) == 1043) {
		$line .= "'" . $row[$k] . "'";
	    } elsif ($rst->ftype($k) == 1082) {
		$line .= "#" . $row[$k] . "#";
	    } elsif ($rst->ftype($k) == 1184) {
		$line .= "'" . $row[$k] . "'";
	    } else {
		$line .= $row[$k];
	    }
	    $comma = ",";
	}
	$line .= ");\n";
	print OUTFILE $line;
    }

    close OUTFILE;
}  #--ExportTable


#--- Method Prolog -----------------------------------------------------------
#  Name: ParseCommandLine
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    ParseCommandLine takes a list of possible options and a boolean 
#    indicating whether the option has a value following, and sets up an 
#    associative array %opt of the values of the options given on the command 
#    line. It removes all the arguments it uses from @ARGV and returns them 
#    in @optArgs.
#
#--- End Method Prolog -------------------------------------------------------
sub ParseCommandLine
{
    my @optval = @_;
    my ($opt, @opts, %valFollows, @newargs, $arg);

    while (@optval) {
	$opt = shift(@optval);
	push(@opts, $opt);
	$valFollows{$opt} = shift(@optval);
    }

    @optArgs = ();
    %opt = ();

    arg: while ($arg = shift(@ARGV)) {
	foreach $opt (@opts) {
	    if ($arg eq $opt) {
		push(@optArgs, $arg);
		if ($valFollows{$opt}) {
		    if (@ARGV == 0) {
			&DisplayUsage();
		    }
		    $opt{$opt} = shift(@ARGV);
		    push(@optArgs, $opt{$opt});
		} else {
		    $opt{$opt} = 1;
		}
		next arg;
	    }
	}
	push(@newargs, $arg);
    }

    @ARGV = @newargs;
}  #--ParseCommandLine


#--- Method Prolog -----------------------------------------------------------
#  Name: Debugln
#
#  Parameters:
#    msg - String message to display to standard error.
#
#  Return Value: None.
#
#  Description:
#    If debug flag is turned on, print out debug, message and line break
#    to standard error. 
#
#--- End Method Prolog -------------------------------------------------------
sub Debugln
{
    my $msg = shift @_;

    if ($SystemPrefs{"debugenable"} == 1) {
	if ($msg ne "") { print STDERR "debug: $msg\n"; }
	else { print STDERR "\n"; }
    }
}  #--Debugln


#--- Method Prolog -----------------------------------------------------------
#  Name: Debug
#
#  Parameters:
#    msg - String message to display to standard error.
#
#  Return Value: None.
#
#  Description:
#    If debug flag is turned on, print out message to standard error.
#
#--- End Method Prolog -------------------------------------------------------
sub Debug
{
    my $msg = shift @_;

    if ($SystemPrefs{"debugenable"} == 1 && $msg ne "") { print STDERR "$msg"; }
}  #--Debug


#--- Method Prolog -----------------------------------------------------------
#  Name: EnableDebugMessages
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Turn on system debuging.
#
#--- End Method Prolog -------------------------------------------------------
sub EnableDebugMessages
{
    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n",
	  "Debug is enabled. Runtime information will be sent to STDERR.\n\n";
    $SystemPrefs{"debugenable"} = 1;
}  #--EnableDebugMessages


#--- Method Prolog -----------------------------------------------------------
#  Name: DisplayUsage
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Display program name and options.
#
#--- End Method Prolog -------------------------------------------------------
sub DisplayUsage
{
    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n",
	  "Usage: " . $SystemPrefs{"app"} . " [OPTION]...\n",
          "Perl interface routines for creating and maintaining the PostgreSQL\n",
          "database used to store Distributed.Net personal proxy log files.\n\n",
	  " -debug    Display debug information while script executes.\n",
          " -version  Display version and copyright information and exit.\n",
          " -help     Print this help and exit.\n\n",
          " -import <table>  Import the given table from the import directory.\n",
          " -importall       Import all tables from the import directory.\n",
          " -export <table>  Export the given table into the export directory.\n",
          " -exportall       Export all tables into the export directory.\n\n",
          " -createdb       Create database.\n",
          " -createtables   Create database tables.\n",
          " -createindexes  Create database indexes for work unit tables.\n",
          " -dropindexes    Drop database indexes for work unit tables.\n",
          " -purgedb        Drop the entire database (warning: deletes table data!).\n\n",
          " -updateuser <datafile>     Update the given participant information.\n", 
          " -retire <datafile>         Retire the given participant.\n",
	  " -blacklist <email>         Blacklist this email address.\n",
	  " -createnetwork <datafile>  Create entry for a network (ip-range)\n\n";

    exit;
}  #--DisplayUsage


#--- Method Prolog -----------------------------------------------------------
#  Name: DisplayVersion
#
#  Parameters: None.
#
#  Return Value: None.
#
#  Description:
#    Display program name and version information.
#
#--- End Method Prolog -------------------------------------------------------
sub DisplayVersion
{
    print "\n" . $SystemPrefs{"appname"} . " v" . $SystemPrefs{"appver"} . "\n",
          "Copyright (C) 1998-2000 GNU General Public License\n",
          "http://ppstats.sourceforge.net/\n\n",
	  "This program is licenced under the GNU General Public Lisence.\n",
          "See License.txt for more information. Comments and suggestions are welcome!\n\n";

    exit;
}  #--DisplayVersion












